"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const apollo_server_core_1 = require("apollo-server-core");
const url_1 = __importDefault(require("url"));
function graphqlNative(options) {
    if (!options) {
        throw new Error('Apollo Server requires options.');
    }
    if (arguments.length > 1) {
        throw new Error(`Apollo Server expects exactly one argument, got ${arguments.length}`);
    }
    const graphqlHandler = (req, res) => __awaiter(this, void 0, void 0, function* () {
        let query;
        if (!req.method)
            throw new Error('Apollo Server expects req.method');
        if (!req.url)
            throw new Error('Apollo Server expects req.url');
        if (req.method === 'POST') {
            try {
                const body = yield new Promise((resolve) => {
                    let rawBody = '';
                    req.on('data', chunk => {
                        rawBody += chunk;
                    });
                    req.on('end', () => {
                        resolve(rawBody);
                    });
                });
                query = JSON.parse(body);
            }
            catch (err) {
                query = undefined;
            }
        }
        else {
            query = url_1.default.parse(req.url, true).query;
        }
        apollo_server_core_1.runHttpQuery([req, res], {
            method: req.method,
            options: options,
            query: query,
            request: apollo_server_core_1.convertNodeHttpToRequest(req)
        }).then(({ graphqlResponse, responseInit }) => {
            if (responseInit.headers) {
                const headers = responseInit.headers;
                Object.keys(headers).forEach(header => {
                    res.setHeader(header, headers[header]);
                });
            }
            // res.setHeader('Content-Type', 'application/json')
            res.setHeader('Content-Length', graphqlResponse.length);
            res.write(graphqlResponse);
            res.end();
        }, (error) => {
            if ('HttpQueryError' !== error.name)
                throw error;
            if (error.headers) {
                Object.keys(error.headers).forEach(header => {
                    res.setHeader(header, error.headers[header]);
                });
            }
            res.statusCode = error.statusCode;
            res.write(error.message);
            res.end();
        });
    });
    return graphqlHandler;
}
exports.graphqlNative = graphqlNative;
//# sourceMappingURL=nativeApollo.js.map